// fluid_project/CDrawBox.cpp

#include "CDrawBox.h"

CPrimitiveBase* create_primitive(
    int type,
    int x1,
    int y1,
    int x2,
    int y2,
    Fl_Color color
    )
{
    switch ( type ) {
    case BTN_RADIO_ID_LINE:
        return new CPrimitiveLine( x1, y1, x2, y2, color );

    case BTN_RADIO_ID_CIRCLE:
        return new CPrimitiveCircle( x1, y1, x2, y2, color );

    case BTN_RADIO_ID_RECTANGLE:
        return new CPrimitiveRectangle( x1, y1, x2, y2, color );
    default:
        assert( !"BAD button id in CMainWindow::click_radio_button()" );
        return nullptr;
    }
}

//------------------------------------
// CPrimitiveBase
//------------------------------------
CPrimitiveBase::CPrimitiveBase( int x1, int y1, int x2, int y2, Fl_Color color )
    : 
x1( x1 ), y1( y1 ), x2( x2 ), y2( y2 ), color( color )
{
}

//------------------------------------
// CPrimitiveLine
//------------------------------------
CPrimitiveLine::CPrimitiveLine( int x1, int y1, int x2, int y2, Fl_Color color )
    : 
CPrimitiveBase( x1, y1, x2, y2, color )
{
}

void CPrimitiveLine::draw()
{
    Fl_Color orig_color = fl_color();
    fl_color( color ); // default color is same as window background
    // note that line is drawn in window coordinates:
    fl_line( x1, y1, x2, y2 ); // diagonal line
    fl_color( orig_color );
}

void CPrimitiveLine::update_x2y2( int x_coord, int y_coord )
{
    x2 = x_coord;
    y2 = y_coord;
}


//------------------------------------
// CPrimitiveCircle
//------------------------------------
CPrimitiveCircle::CPrimitiveCircle( int x1, int y1, int x2, int y2, Fl_Color color )
    : 
CPrimitiveBase( x1, y1, x2, y2, color )
{
}

void CPrimitiveCircle::draw()
{
    //std::cout << "CPrimitiveCircle::draw()" << x1 << y1 << x2 << y2 << "\n";
    Fl_Color orig_color = fl_color();
    fl_color( color ); // default color is same as window background
    fl_arc( x1, y1, x2 - x1, y2 - y1, 0, 360 );
    fl_color( orig_color );
}

void CPrimitiveCircle::update_x2y2( int x_coord, int y_coord )
{
    x2 = x_coord;
    y2 = y_coord;
}

//------------------------------------
// CPrimitiveRectangle
//------------------------------------

CPrimitiveRectangle::CPrimitiveRectangle( int x1, int y1, int x2, int y2, Fl_Color color )
    : 
CPrimitiveBase( x1, y1, x2, y2, color )
{
}

void CPrimitiveRectangle::draw()
{
    Fl_Color orig_color = fl_color();
    fl_color( color ); // default color is same as window background
    fl_rect( x1, y1, x2 - x1, y2 - y1 );
    fl_color( orig_color );
}

void CPrimitiveRectangle::update_x2y2( int x_coord, int y_coord )
{
    x2 = x_coord;
    y2 = y_coord;
}

//------------------------------------
// CDrawBox
//------------------------------------

void CDrawBox::draw() {
    // std::cout << "CDrawBox::draw()\n";
    // Make sure Fl_Box is drawn:
    Fl_Box::draw();
    draw_clear();
    draw_border();
    draw_primitives();
}

void CDrawBox::draw_primitives()
{
    for ( size_t idx = 0; idx < vect_primitives.size(); ++idx ) {
        vect_primitives[ idx ]->draw();
    }
}

void CDrawBox::draw_border()
{
    Fl_Color orig_color = fl_color();
    fl_color( border_color );
    // A---B
    // |   |
    // C---D
    fl_line( x(), y(),             x() + w() - 1, y()           ); // A-B
    fl_line( x() + w() - 1, y(),   x() + w() - 1, y() + h() - 1 ); // B-D
    fl_line( x(), y() + h() - 1,   x() + w() - 1, y() + h() - 1 ); // C-D
    fl_line( x(), y(),             x(), y() + h() - 1           ); // A-C
    fl_color( orig_color );
}

void CDrawBox::draw_clear()
{
    Fl_Color orig_color = fl_color();
    fl_color( FL_BACKGROUND_COLOR ); // the default background color
    // A---B
    // |   |
    // C---D
    fl_rectf( x(), y(), x() + w() - 1, y() + h() - 1 ); // filled rectangle
    fl_color( orig_color );
}

void CDrawBox::use_primitive_type( int type )
{
    primitive_type = type;
}

int CDrawBox::handle( int event_ )
{
    //if ( event_ != FL_MOVE ) std::cout << "CDrawBox::handle() " << fl_eventnames[ event_ ] << " " << vect_primitives.size() << "\n";
    const int HANDLE_EVENT_HERE = 1; // indicates that we want to follow-up on this event
    const int HANDLE_EVENT_ELSEWHERE = 0; // indicates that we want don't care about this event
    int button_code = 0; // mouse button id
    int x_coord = 0;     // X coord
    int y_coord = 0;     // Y coord
    int result = Fl_Box::handle( event_ );

    switch ( event_ ) {
    case FL_PUSH: // mouse button click
        button_code = Fl::event_button(); // FL_LEFT_MOUSE FL_MIDDLE_MOUSE FL_RIGHT_MOUSE
        push_x_coord = Fl::event_x();
        push_y_coord = Fl::event_y();
        result = HANDLE_EVENT_HERE; // we want to process mouse events
        break;
    case FL_DRAG:
        // call int Fl::event_state() to find out whether ALT/SHIFT/CTRL etc. are pressed
        x_coord = Fl::event_x();
        y_coord = Fl::event_y();
        if ( push_x_coord != DRAW_ILLEGAL_COORDINATE ) {
            // first drag event since FL_PUSH
            vect_primitives.push_back(
                std::unique_ptr< CPrimitiveBase >( create_primitive(
                primitive_type,
                push_x_coord, push_y_coord, x_coord, y_coord, FL_BLACK
                ) )
                );
            push_x_coord = DRAW_ILLEGAL_COORDINATE;
            push_y_coord = DRAW_ILLEGAL_COORDINATE;
        } else {
            // otherwise reposition the second end of the line segment
            assert( vect_primitives.size() );
            vect_primitives[ vect_primitives.size() - 1 ]->update_x2y2( x_coord, y_coord );
        }
        result = HANDLE_EVENT_HERE; // we want to process mouse events
        Fl_Box::redraw();
        break;
    case FL_RELEASE:
        // the drag is over
        // make sure we reset our coordinates:
        push_x_coord = DRAW_ILLEGAL_COORDINATE;
        push_y_coord = DRAW_ILLEGAL_COORDINATE;
        break;
    case FL_ENTER:
        // mouse entered the box
        border_color = FL_RED;
        Fl_Box::redraw();
        break;
    case FL_LEAVE:
        // mouse left the box
        border_color = FL_BLUE;
        Fl_Box::redraw();
        break;
    }
    return result;
}
